/**********************************************************************
 *
 *  cvt_main.c
 *      Terascan Data Format to Network Common Data Format convesion
 *
 *      main program
 *
 * ADM 08 Sep 92
 **********************************************************************/
#include <errno.h>
#include <stdio.h>
#include <sys/param.h>
#include <malloc.h>

#include "netcdf.h"
#include "cvt_defs.h"
#include "gp.h"
/**********************************************************************/
/*
 * global declarations
 */
/*
 * external declarations
 */
extern int ncerr;
extern void mygperr();	/* Exits on all errors except open */
/*   char inname[]="/home/local/migs/amy/f10.91191.0403.rr"; */

/**********************************************************************/

nc_type
netcdftype(type)  /* converts tdf type to netcdf type */
int type;
{
   switch(type) {
/*
    case GP_BYTE:
      return(NC_BYTE);
      break;
*/
    case GP_BYTE:
      return(NC_CHAR);
      break;

    case GP_SHORT:
      return(NC_SHORT);
      break;
    case GP_LONG:
/*    case GP_INT:
*/
      return(NC_LONG);
      break;
    case GP_FLOAT:
/*    case GP_REAL:
*/
      return(NC_FLOAT);
      break;
    case GP_DOUBLE:
      return(NC_DOUBLE);
      break;
    default:
      printf("netcdftype ERROR!\n");
      return(NC_BYTE);
      break;
   }
} /* end netcdftype() */

/**********************************************************************/
int
main(argc, argv)
int argc;  char **argv;
{			/* create tdfto.cdf */

   /*
    * =============   local declarations   =============
    */
   SETP set;
   DIMP dim;
   VARP var;
   ATTP att;

   int
     cdfid,			 /* netCDF file id */
     dimid, 
     dim_id[32],  /* dimension IDs created by ncdimdef(),
                       acquired by ncdimid() */
     i,
     rc,                         /* return code */
     num_channels,
     varid,
     ch_id[32];
   int
     file,
     nfiles,
     ngood;
   char
     *dir,
     *dirfile(),
     *inname,
     *infile,
     *outname,
     path[MAXPATHLEN],
     outbuf[MAXPATHLEN];

   long start[MAX_VAR_DIMS];
   long count[MAX_VAR_DIMS];

   char *valuesptr;

   bzero(start, sizeof(start));
   bzero(count, sizeof(count));
   bzero(dim_id, sizeof(dim_id));
   bzero(ch_id, sizeof(ch_id));

/*---------------------------------*/
  nfiles = --argc;	/* Skip command name */
  argv++;

  if (nfiles < 2) {
    printf("\nUsage: 1) tdf.exe path\\infile path\\outfile\n");
    printf("  or   2) tdf.exe infile1 infile2 ... infileN dir\n");
    printf("          (where infileN is in dir and outfileN\n");
    printf("           will be dir\\infileN.cdf)\n\n");
    exit(1);
  }

  dir = (isdir(argv[nfiles -1]) > 0) ? argv[nfiles -1] : NULL;

  if (nfiles > 2 && dir == NULL) {
    printf("tdf.exe: %s is not a directory\n", argv[nfiles -1]);
    exit(1);
  }
  gperr = mygperr;	/* Use own error handler */

  /* Loop through files */
  for (ngood = file = 0; file < nfiles -1; file++) {

    inname = argv[file];
    if (dir) {	/* Synthesize name */
      inname = dirfile(dir, inname, path);
      strcpy(outbuf,path);
      strcat(outbuf,".cdf");
      outname=outbuf;
    } else outname = argv[nfiles -1];

   /* open terascan file */

    set = gpopen(inname, GP_READ);
    if (set == NULL) {
        printf("could not find %s\n", inname);
        exit();
    }

   
/*---------------------------------*/
   /* create a new netCDF file and enter define mode */

   cdfid = nccreate(outname, NC_CLOBBER);
   if (cdfid == -1) {
        printf("nccreate error:  ncerr=%d\n", ncerr);
        exit();
   }

/*---------------------------------*/

    /* define netCDF dimensions */
/* -- example --
TDF dimensions:

name	: line
size	: 1200

name	: sample
size	: 1200

name	: dim_3
size	: 103
*/
    if (set->dim != NULL) {
      printf("\ndimensions:\n");
      for (i=0, dim = set->dim; dim != NULL; dim = dim->next, i++) {
        /*
	if (dim->var != NULL)
	    printf("var\t: %s\n", ((VARP)(dim->var))->name);
         */
        /* define dimensions: size MUST be long */
        dim_id[i] = ncdimdef(cdfid, dim->name, (long)(dim->size));
        if (dim_id[i] == -1) {
            printf("ncdimdef error:  ncerr=%d\n", ncerr);
            exit();
        }
        count[i] = (long)(dim->size);
      }
    }
   
/*---------------------------------*/

    /* define netCDF VARIABLES */
printf("\nvariables:\n");

    if (set->var != NULL) {
        for (var = set->var; var != NULL; var = var->next) {

/*
TDF variables:

name	: avhrr_ch1
units	: albedo*100%
type	: short
dims	: line sample
scale	: 0.01
offset	: 0
badval	: -32768
usemin	: -32768
usemax	: 32767
*/

        
            ch_id[i] = ncvardef(cdfid, var->name, netcdftype(var->type),
                var->ndims, dim_id);
            if (ch_id[i] == -1) {
                printf("ncvardef error:  ncerr=%d\n", ncerr);
                exit();
            }

            rc = ncattput(cdfid, ch_id[i], UNITS, NC_CHAR,
                strlen(var->units) + 1, (void *) var->units);
            if (rc == -1) {
                printf("ncattput error: ncerr=%d\n", ncerr);
                exit();
            }
            rc = ncattput (cdfid, ch_id[i], SCALE_FACTOR, NC_DOUBLE, 1,
                (void *) &(var->scale));
            if (rc == -1) {
                printf("ncattput error: ncerr=%d\n", ncerr);
                exit();
            }
            var->usemin = 1 + var->usemin;
            rc = ncattput (cdfid, ch_id[i], VALID_MIN, NC_DOUBLE, 1,
                (void *) &(var->usemin));
            if (rc == -1) {
                printf("ncattput error: ncerr=%d\n", ncerr);
                exit();
            }
            rc = ncattput (cdfid, ch_id[i], VALID_MAX, NC_DOUBLE, 1,
                (void *) &(var->usemax));
            if (rc == -1) {
                printf("ncattput error: ncerr=%d\n", ncerr);
                exit();
            }
            rc = ncattput (cdfid, ch_id[i], ADD_OFFSET, NC_DOUBLE, 1,
                (void *) &(var->offset));
            if (rc == -1) {
                printf("ncattput error: ncerr=%d\n", ncerr);
                exit();
            }

            rc = ncattput (cdfid, ch_id[i], FILLVALUE, NC_DOUBLE, 1,
                (void *) &(var->badval));
            if (rc == -1) {
                printf("ncattput error: ncerr=%d\n", ncerr);
                exit();
            }

        } /* end for (var = set->var) */
    }/* end if (set->var != NULL) */
    num_channels=i;
/*---------------------------------*/

    /* define attributes */

    if (set->att != NULL) {
        printf("\nattributes:\n");
        rc = ncattput(cdfid, NC_GLOBAL, DATE_DESC,
            NC_CHAR, strlen(DATE_UNIT) + 1, DATE_UNIT);
        if (rc == -1) {
            printf("ncattput error: ncerr=%d\n", ncerr);
            exit();
        }
        rc = ncattput(cdfid, NC_GLOBAL, ANGLE_DESC,
            NC_CHAR, strlen(ANGLE_UNIT) + 1, ANGLE_UNIT);
        if (rc == -1) {
            printf("ncattput error: ncerr=%d\n", ncerr);
            exit();
        }

        for (att = set->att; att != NULL; att = att->next) {
            rc = ncattput (cdfid, NC_GLOBAL, att->name,
                netcdftype(att->type), att->size, att->value);
            if (rc == -1) {
                printf("ncattput error: ncerr=%d\n", ncerr);
                exit();
            }
        } /* end for */
    } /* end if */


/*---------------------------------*/

   /* leave define mode */
   rc = ncendef (cdfid);
   if (rc == -1) printf("ncendef error: ncerr = %d\n",ncerr);
/*---------------------------------*/
  /* put values */

  /* call ncvarput */
  for (var = set->var; var != NULL; var = var->next) {
      varid = ncvarid(cdfid, var->name);
      if (copyvardata(cdfid, varid, var, GP_ORIGIN) < 0) {
        perror(outname);
	exit(1);
      }
  }


/*---------------------------------*/
    (void)gpclose(set);
/*---------------------------------*/

/*
ncendef: var ols_infrared: _FillValue type mismatch
*/
   rc = ncclose (cdfid);
   if (rc == -1) {
       printf("ncclose error: ncerr=%d\n", ncerr);
   }
    ngood++;

  }  /* End file loop */

  exit((ngood == nfiles -1) ? 0 : 1);

} /* end main() */

/**********************************************************************/

int copyvardata(cdfid, varid, invar, where)
int cdfid;
int varid;
VARP invar;
int *where;		/* Output hypercube origin */
{
  int start[GP_VAR_DIMS];
  int count[GP_VAR_DIMS];
  int rc, i, iolen, bound[GP_VAR_DIMS], ndiff;
  char *data;

  bzero(start, sizeof(start));
  bzero(count, sizeof(count));
  bzero(bound, sizeof(bound));

  /* Accumulate total IO length */

  iolen = gptypelen(invar->type);

  for (i = 0; i < invar->ndims; i++) {
    count[i] = bound[i] = invar->dim[i]->size;
  }
  iolen *= count[1];
  if (iolen == 0) return(0);
  count[0] = 1;

  /* Determine IO length that fits in memory */

  if (iolen/bound[0] > MAXMEM) {		/* DUMB VERSION */
    errno = ENOMEM;
    return(-1);
  }

  if (iolen > MAXMEM) {
    iolen /= bound[0];
    count[0] = MAXMEM / iolen;
    iolen *= count[0];
  }

  /* Allocate memory */
  data = malloc(iolen);
  if (data == (char *)0) return(-1);

  /* Read and write data */
  for (start[0] = 0; start[0] < bound[0]; start[0] += count[0]) {
    if (gpgetvar(invar, start, count, data) < 0) break;
      rc = ncvarput(cdfid, varid, start, count, (void *)data);
      if (rc == -1) {
          printf("ncvarput error: ncerr = %d\n",ncerr);
          break;
      }
  }
  free(data);
  if (start[0] < bound[0]) return(-1);
  return(0);
} /* end copyvardata() */

/**********************************************************************/
/*                       end of file cvt_main.c                       */
/**********************************************************************/
